@@ -1,16 +1,29 @@
 # -*- coding: utf-8 -*-
+# import modules
+import os, sys, json
+from threading import Timer
+from collections import OrderedDict
+from datetime import datetime
+from glob import glob
+from copy import copy
+import subprocess
+from flask import Flask, Blueprint, redirect, request, jsonify
+from flask_socketio import SocketIO, emit
+
 from . import __path__
+from .models import View
 from .logger import Logger
 
 class Server():
-    def __init__(self, appDir:str):
-        import os, sys, json
-        from datetime import datetime
-        from flask import Flask, request, jsonify
-        from flask_socketio import SocketIO, emit
+    def __init__(self, appDir:str, enableLogging:bool = True, webview = None):
+        # check logging
+        self.__logging = enableLogging
 
         # variables
-        Logger.info("Prepare Server to run app...")
+        if self.__logging:
+            Logger.info("Prepare Server to run app...")
+
+        self.__webview = webview
         self.__appDir = os.path.abspath(appDir)
         self.__session = {
             "app": {
@@ -35,15 +48,86 @@ class Server():
         self.__app.secret_key = "pyvuejsApp"
         self.__socketio = SocketIO(self.__app)
 
+        # view routing points
+        if self.__logging:
+            Logger.info("Setting view/component routing points...")
+
+        @self.__app.route("/views/<viewName>")
+        def showView(viewName):
+            if viewName in self.__session["app"]["views"].keys():
+                viewId = request.remote_addr
+                if not viewId in self.__session["data"]["mpa"].keys():
+                    self.__session["data"]["mpa"][viewId] = {}
+
+                if not viewId in self.__session["app"]["mpa"].keys():
+                    self.__session["app"]["mpa"][viewId] = OrderedDict()
+
+                if not viewName in self.__session["app"]["mpa"][viewId].keys():
+                    self.__session["app"]["mpa"][viewId][viewName] = copy(self.__session["app"]["views"][viewName])
+
+                    for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
+                        for varName, var in model.sessions.items():
+                            self.__session["data"]["mpa"][viewId][varName] = var
+
+                        if "load" in model.events.keys():
+                            model.events["load"](self.__session["data"]["mpa"][viewId])
+
+                for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
+                    if "show" in model.events.keys():
+                        model.events["show"](self.__session["data"]["mpa"][viewId])
+
+                return self.__session["app"]["mpa"][viewId][viewName].render(viewId)
+            else:
+                viewNames = list(self.__session["app"]["views"].keys())
+                if viewName == "def@ltWindowed" and len(viewNames) > 0:
+                    if "main" in viewNames:
+                        return redirect("/views/main")
+                    else:
+                        return redirect("/views/{}".format(viewNames[0]))
+                else:
+                    return ""
+
+        @self.__app.route("/components/<viewName>")
+        def showComponent(viewName):
+            if viewName in self.__session["app"]["components"].keys():
+                viewId = request.remote_addr
+                if not viewId in self.__session["data"]["mpa"].keys():
+                    self.__session["data"]["mpa"][viewId] = {}
+
+                if not viewId in self.__session["app"]["mpa"].keys():
+                    self.__session["app"]["mpa"][viewId] = OrderedDict()
+
+                if not viewName in self.__session["app"]["mpa"][viewId].keys():
+                    self.__session["app"]["mpa"][viewId][viewName] = copy(self.__session["app"]["components"][viewName])
+
+                    for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
+                        for varName, var in model.sessions.items():
+                            self.__session["data"]["mpa"][viewId][varName] = var
+
+                        if "load" in model.events.keys():
+                            model.events["load"](self.__session["data"]["mpa"][viewId])
+
+                for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
+                    if "show" in model.events.keys():
+                        model.events["show"](self.__session["data"]["mpa"][viewId])
+
+                return self.__session["app"]["mpa"][viewId][viewName].render(viewId)
+            else:
+                return ""
+
         # function routing points
-        Logger.info("Setting function routing points...")
+        if self.__logging:
+            Logger.info("Setting function routing points...")
+
         @self.__app.before_first_request
         def onFirstRequest():
             pass
 
         @self.__app.route("/shutdown", methods = ["POST"])
         def onShutdown():
-            Logger.info("App is shutting down...")
+            if self.__logging:
+                Logger.info("App is shutting down...")
+
             sys.exit()
 
         @self.__app.route("/session/upload", methods = ["POST"])
@@ -63,54 +147,63 @@ class Server():
         def onSessionDownload():
             res = json.loads(request.data)
 
-            # check expiredTime
-            for viewId, updateTime in self.__session["app"]["updatedTime"].items():
-                if not res["id"] == viewId:
-                    if (datetime.now() - updateTime).total_seconds() >= self.__session["config"]["expiredTime"]:
-                        self.__session["app"]["mpa"].pop(viewId)
-                        self.__session["data"]["mpa"].pop(viewId)
+            try:
+                # check expiredTime
+                for viewId, updateTime in self.__session["app"]["updatedTime"].items():
+                    if not res["id"] == viewId:
+                        if (datetime.now() - updateTime).total_seconds() >= self.__session["config"]["expiredTime"]:
+                            self.__session["app"]["mpa"].pop(viewId)
+                            self.__session["data"]["mpa"].pop(viewId)
 
-            resp = {
-                "id": self.__session["data"]["excludes"]["id"],
-                "excludeName": self.__session["data"]["excludes"]["name"],
-                "sessions": {}
-            }
-            for view in self.__session["app"]["mpa"][resp["id"]].values():
-                sessionSet = {}
-                for modelName in view.models.keys():
-                    sessionSet[modelName] = self.__session["data"]["mpa"][resp["id"]]
+                resp = {
+                    "id": self.__session["data"]["excludes"]["id"],
+                    "excludeName": self.__session["data"]["excludes"]["name"],
+                    "sessions": {}
+                }
+                for view in self.__session["app"]["mpa"][resp["id"]].values():
+                    sessionSet = {}
+                    for modelName in view.models.keys():
+                        sessionSet[modelName] = self.__session["data"]["mpa"][resp["id"]]
 
-                resp["sessions"][view.name] = sessionSet
+                    resp["sessions"][view.name] = sessionSet
 
-            return jsonify(resp)
+                return jsonify(resp)
+            except KeyError:
+                return jsonify({
+                    "id": "",
+                    "excludeName": "",
+                    "sessions": {}
+                })
 
         # socket points
-        Logger.info("Setting socket routing points...")
+        if self.__logging:
+            Logger.info("Setting socket routing points...")
+
         @self.__socketio.on("feedbackPY", namespace = "/pyvuejsSocket")
         def onFeedback(res):
             print(res)
 
         @self.__socketio.on("initViewPY", namespace = "/pyvuejsSocket")
         def onInitView(res):
-            # if res["id"] in self.__session["app"]["mpa"].keys():
-            view = self.__session["app"]["mpa"][res["id"]][res["name"]]
-            emit("initViewJS", {
-                "id": res["id"],
-                "name": res["name"],
-                "session": self.__session["data"]["mpa"][res["id"]],
-                "data": {
-                    modelName: model.variables
-                    for modelName, model in view.models.items()
-                },
-                "computes": {
-                    modelName: list(model.computes.keys())
-                    for modelName, model in view.models.items()
-                },
-                "methods": {
-                    modelName: list(model.methods.keys())
-                    for modelName, model in view.models.items()
-                }
-            })
+            if res["id"] in self.__session["app"]["mpa"].keys():
+                view = self.__session["app"]["mpa"][res["id"]][res["name"]]
+                emit("initViewJS", {
+                    "id": res["id"],
+                    "name": res["name"],
+                    "session": self.__session["data"]["mpa"][res["id"]],
+                    "data": {
+                        modelName: model.variables
+                        for modelName, model in view.models.items()
+                    },
+                    "computes": {
+                        modelName: list(model.computes.keys())
+                        for modelName, model in view.models.items()
+                    },
+                    "methods": {
+                        modelName: list(model.methods.keys())
+                        for modelName, model in view.models.items()
+                    }
+                })
 
         @self.__socketio.on("compute", namespace = "/pyvuejsSocket")
         def onCompute(res):
@@ -148,23 +241,23 @@ class Server():
                 "session": self.__session["data"]["mpa"][res["id"]]
             })
 
-        Logger.info("Routing points are ready!\n")
+        if self.__logging:
+            Logger.info("Routing points are ready!\n")
 
         self.__interpret_appDir()
-        Logger.info("Server is ready!\n")
+        if self.__logging:
+            Logger.info("Server is ready!\n")
 
     def __interpret_appDir(self):
-        import os, sys
-        from glob import glob
-        from importlib import import_module
-        from flask import Blueprint, session
-        from .models import View
+        if self.__logging:
+            Logger.info("Interpreting app...")
 
-        Logger.info("Interpreting app...")
         if os.path.exists(self.__appDir):
             sys.path.append(self.__appDir)
 
-            Logger.info("Interpreting view files...")
+            if self.__logging:
+                Logger.info("Interpreting view files...")
+
             for pvPath in glob(os.path.join(self.__appDir, "views", "*.pvue")):
                 pvInfo = self.__interpret_pyvue(pvPath)
                 
@@ -172,7 +265,8 @@ class Server():
                     pvInfo["name"], pvInfo["prefix"],
                     pvInfo["resource"], pvInfo["style"], pvInfo["script"],
                     pvInfo["template"],
-                    pvInfo["model"]
+                    pvInfo["model"],
+                    self.__webview
                 )
 
                 if pvInfo["prefix"] == "view":
@@ -180,10 +274,14 @@ class Server():
                 elif pvInfo["prefix"] == "component":
                     self.__session["app"]["components"][pvInfo["name"]] = view
 
-                Logger.info("{}.pvue has been interpreted".format(pvInfo["name"]))
-            Logger.info("Finished!\n")
+                if self.__logging:
+                    Logger.info("{}.pvue has been interpreted".format(pvInfo["name"]))
 
-            Logger.info("Linking static files to server...")
+            if self.__logging:
+                Logger.info("Finished!\n")
+
+            if self.__logging:
+                Logger.info("Linking static files to server...")
             staticDir = os.path.join(self.__appDir, "static")
             if os.path.exists(staticDir):
                 self.__app.register_blueprint(
@@ -193,9 +291,11 @@ class Server():
                         static_folder = staticDir
                     )
                 )
-            Logger.info("Finished!\n")
+            if self.__logging:
+                Logger.info("Finished!\n")
         
-        Logger.info("App has been ready!\n")
+        if self.__logging:
+            Logger.info("App has been ready!\n")
 
     def __interpret_pyvue(self, pvFile) -> dict:
         import os
@@ -304,73 +404,68 @@ def {0}():
 '''
 """.format(fName, view.render())
 
-    def start(self, host:str = "0.0.0.0", port:int = 8000):
-        import os
-        from flask import request, session
-        from copy import deepcopy
-
-        @self.__app.route("/views/<viewName>")
-        def showView(viewName):
-            if viewName in self.__session["app"]["views"].keys():
-                viewId = request.remote_addr
-                if not viewId in self.__session["data"]["mpa"].keys():
-                    self.__session["data"]["mpa"][viewId] = {}
-
-                if not viewId in self.__session["app"]["mpa"].keys():
-                    self.__session["app"]["mpa"][viewId] = {}
-
-                if not viewName in self.__session["app"]["mpa"][viewId].keys():
-                    self.__session["app"]["mpa"][viewId][viewName] = deepcopy(self.__session["app"]["views"][viewName])
-
-                    for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
-                        for varName, var in model.sessions.items():
-                            self.__session["data"]["mpa"][viewId][varName] = var
-
-                        if "load" in model.events.keys():
-                            model.events["load"](self.__session["data"]["mpa"][viewId])
-
-                for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
-                    if "show" in model.events.keys():
-                        model.events["show"](self.__session["data"]["mpa"][viewId])
-
-                return self.__session["app"]["mpa"][viewId][viewName].render(viewId)
-            else:
-                return ""
-
-        @self.__app.route("/components/<viewName>")
-        def showComponent(viewName):
-            if viewName in self.__session["app"]["components"].keys():
-                viewId = request.remote_addr
-                if not viewId in self.__session["data"]["mpa"].keys():
-                    self.__session["data"]["mpa"][viewId] = {}
-
-                if not viewId in self.__session["app"]["mpa"].keys():
-                    self.__session["app"]["mpa"][viewId] = {}
-
-                if not viewName in self.__session["app"]["mpa"][viewId].keys():
-                    self.__session["app"]["mpa"][viewId][viewName] = deepcopy(self.__session["app"]["components"][viewName])
-
-                    for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
-                        for varName, var in model.sessions.items():
-                            self.__session["data"]["mpa"][viewId][varName] = var
-
-                        if "load" in model.events.keys():
-                            model.events["load"](self.__session["data"]["mpa"][viewId])
-
-                for model in self.__session["app"]["mpa"][viewId][viewName].models.values():
-                    if "show" in model.events.keys():
-                        model.events["show"](self.__session["data"]["mpa"][viewId])
-
-                return self.__session["app"]["mpa"][viewId][viewName].render(viewId)
-            else:
-                return ""
-
+    def start(self, host:str = "0.0.0.0", port:int = 8080):
         try:
             if host in ("127.0.0.1", "localhost"):
                 host = "0.0.0.0"
 
-            Logger.info("Server started on \"{}:{}\"".format(host, port))
-            Logger.info("Please check Devtool to show data transfers")
+            if self.__logging:
+                Logger.info("Server started on \"http://{}:{}\"".format(host, port))
+                Logger.info("Please check Devtool to show data transfers")
             self.__socketio.run(self.__app, host = host, port = port)
         except KeyboardInterrupt:
             os.remove(os.path.join(self.__appDir, ".state"))
+
+class WindowedServer():
+    def __init__(self, enableLogging:bool = True):
+        from PySide2.QtWidgets import QApplication
+        from PySide2.QtWebEngineWidgets import QWebEngineView
+
+        self.__logging = enableLogging
+
+        self.__qtApp = QApplication(sys.argv)
+        self.__window = QWebEngineView()
+        self.__window.loadFinished.connect(self.__onLoaded)
+        self.__window.closeEvent = self.__onClosed
+
+    def __onLoaded(self, finished:bool):
+        self.__window.showNormal()
+
+        if self.__logging:
+            Logger.info("Webview is loaded")
+
+    def __onClosed(self, ev):
+        if self.__logging:
+            Logger.info("Shutting down background server...")
+
+        subprocess.Popen([sys.executable, ".\manage.py", "stop"])
+
+    def __start_server_background(self, appDir:str, port:int):
+        Server(appDir, False, self.__window).start("127.0.0.1", port)
+
+    def start(self, appDir:str, port:int = 8080, window_size:list = [900, 600]):
+        from PySide2.QtGui import QIcon
+        from PySide2.QtCore import QUrl, QThread
+
+        if self.__logging:
+            Logger.info("Start server on background...")
+
+        os.chdir(appDir)
+        self.__timer = Timer(1, self.__start_server_background, (appDir, port))
+        self.__timer.start()
+        # subprocess.Popen([sys.executable, ".\manage.py", "run", "--host={}".format(host), "--port={}".format(port), "--logging=disable", "--mode=server"])
+
+        if self.__logging:
+            Logger.info("Setting up webview...")
+
+        appIcon = os.path.join(appDir, "static", "favicon.png")
+        if not os.path.exists(appIcon):
+            appIcon = os.path.join(__path__[0], "static", "favicon.png")
+
+        self.__qtApp.setWindowIcon(QIcon(appIcon))
+
+        self.__window.setWindowTitle(os.path.basename(appDir))
+        self.__window.load(QUrl("http://127.0.0.1:{}/views/def@ltWindowed".format(port)))
+        self.__window.resize(window_size[0], window_size[1])
+
+        sys.exit(self.__qtApp.exec_())
